@@ -3,6 +3,8 @@
 package app
 
 import (
+	clusterv1client "github.com/open-cluster-management/api/client/cluster/clientset/versioned"
+	clusterv1informers "github.com/open-cluster-management/api/client/cluster/informers/externalversions"
 	"github.com/stolostron/multicloud-operators-foundation/pkg/proxyserver/api"
 	"github.com/stolostron/multicloud-operators-foundation/pkg/proxyserver/getter"
 	genericapiserver "k8s.io/apiserver/pkg/server"
@@ -14,7 +16,9 @@ type ProxyServer struct {
 }
 
 func NewProxyServer(
+	client clusterv1client.Interface,
 	informerFactory informers.SharedInformerFactory,
+	clusterInformer clusterv1informers.SharedInformerFactory,
 	apiServerConfig *genericapiserver.Config,
 	proxyGetter *getter.ProxyServiceInfoGetter,
 	logGetter getter.ConnectionInfoGetter) (*ProxyServer, error) {
@@ -23,7 +27,7 @@ func NewProxyServer(
 		return nil, err
 	}
 
-	if err := api.Install(proxyGetter, logGetter, apiServer); err != nil {
+	if err := api.Install(proxyGetter, logGetter, apiServer, client, informerFactory, clusterInformer); err != nil {
 		return nil, err
 	}
 
