@@ -68,3 +68,12 @@ func (agg aggregate) visit(f func(err error) bool) bool {
 
 	return false
 }
+
+//appendErrors append errs, return appended result
+func AppendErrors(errsList ...[]error) []error {
+	var returnErr []error
+	for _, errs := range errsList {
+		returnErr = append(returnErr, errs...)
+	}
+	return returnErr
+}
