@@ -2,14 +2,13 @@ package createcluster
 
 import (
 	"bytes"
-	"encoding/json"
 	"fmt"
 	"io/ioutil"
-	"net/http"
 	"os"
 	"os/user"
 	"path/filepath"
 	"strings"
+	"time"
 
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
@@ -20,20 +19,18 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/cli-runtime/pkg/printers"
 	"k8s.io/client-go/kubernetes/scheme"
-	"k8s.io/client-go/tools/clientcmd"
-	"sigs.k8s.io/controller-runtime/pkg/client/config"
 
+	"github.com/openshift/hive/apis"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	"github.com/openshift/hive/contrib/pkg/utils"
 	awsutils "github.com/openshift/hive/contrib/pkg/utils/aws"
 	azurecredutil "github.com/openshift/hive/contrib/pkg/utils/azure"
 	gcputils "github.com/openshift/hive/contrib/pkg/utils/gcp"
 	openstackutils "github.com/openshift/hive/contrib/pkg/utils/openstack"
 	ovirtutils "github.com/openshift/hive/contrib/pkg/utils/ovirt"
-	"github.com/openshift/hive/pkg/apis"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
 	"github.com/openshift/hive/pkg/clusterresource"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/gcpclient"
-	"github.com/openshift/hive/pkg/resource"
 	"github.com/openshift/installer/pkg/validate"
 )
 
@@ -117,56 +114,65 @@ var (
 
 // Options is the set of options to generate and apply a new cluster deployment
 type Options struct {
-	Name                     string
-	Namespace                string
-	SSHPublicKeyFile         string
-	SSHPublicKey             string
-	SSHPrivateKeyFile        string
-	BaseDomain               string
-	PullSecret               string
-	PullSecretFile           string
-	Cloud                    string
-	CredsFile                string
-	ClusterImageSet          string
-	ReleaseImage             string
-	ReleaseImageSource       string
-	DeleteAfter              string
-	ServingCert              string
-	ServingCertKey           string
-	UseClusterImageSet       bool
-	ManageDNS                bool
-	Output                   string
-	IncludeSecrets           bool
-	InstallOnce              bool
-	UninstallOnce            bool
-	SimulateBootstrapFailure bool
-	WorkerNodesCount         int64
-	CreateSampleSyncsets     bool
-	ManifestsDir             string
-	Adopt                    bool
-	AdoptAdminKubeConfig     string
-	AdoptInfraID             string
-	AdoptClusterID           string
-	AdoptAdminUsername       string
-	AdoptAdminPassword       string
-	MachineNetwork           string
-	Region                   string
-	Labels                   []string
-	SkipMachinePools         bool
-	AdditionalTrustBundle    string
+	Name                              string
+	Namespace                         string
+	SSHPublicKeyFile                  string
+	SSHPublicKey                      string
+	SSHPrivateKeyFile                 string
+	BaseDomain                        string
+	PullSecret                        string
+	PullSecretFile                    string
+	BoundServiceAccountSigningKeyFile string
+	Cloud                             string
+	CredsFile                         string
+	CredentialsModeManual             bool
+	ClusterImageSet                   string
+	ReleaseImage                      string
+	ReleaseImageSource                string
+	DeleteAfter                       string
+	HibernateAfter                    string
+	HibernateAfterDur                 *time.Duration
+	ServingCert                       string
+	ServingCertKey                    string
+	UseClusterImageSet                bool
+	ManageDNS                         bool
+	Output                            string
+	IncludeSecrets                    bool
+	InstallOnce                       bool
+	UninstallOnce                     bool
+	SimulateBootstrapFailure          bool
+	WorkerNodesCount                  int64
+	CreateSampleSyncsets              bool
+	ManifestsDir                      string
+	Adopt                             bool
+	AdoptAdminKubeConfig              string
+	AdoptInfraID                      string
+	AdoptClusterID                    string
+	AdoptAdminUsername                string
+	AdoptAdminPassword                string
+	MachineNetwork                    string
+	Region                            string
+	Labels                            []string
+	Annotations                       []string
+	SkipMachinePools                  bool
+	AdditionalTrustBundle             string
+	CentralMachineManagement          bool
+	Internal                          bool
 
 	// AWS
-	AWSUserTags []string
+	AWSUserTags    []string
+	AWSPrivateLink bool
 
 	// Azure
 	AzureBaseDomainResourceGroupName string
 
 	// OpenStack
-	OpenStackCloud           string
-	OpenStackExternalNetwork string
-	OpenStackMasterFlavor    string
-	OpenStackComputeFlavor   string
-	OpenStackAPIFloatingIP   string
+	OpenStackCloud             string
+	OpenStackExternalNetwork   string
+	OpenStackMasterFlavor      string
+	OpenStackComputeFlavor     string
+	OpenStackAPIFloatingIP     string
+	OpenStackIngressFloatingIP string
 
 	// VSphere
 	VSphereVCenter          string
@@ -188,11 +194,12 @@ type Options struct {
 	OvirtCACerts         string
 
 	homeDir string
+	log     log.FieldLogger
 }
 
 // NewCreateClusterCommand creates a command that generates and applies cluster deployment artifacts.
 func NewCreateClusterCommand() *cobra.Command {
-	opt := &Options{}
+	opt := &Options{log: log.WithField("command", "create-cluster")}
 
 	opt.homeDir = "."
 
@@ -205,7 +212,7 @@ func NewCreateClusterCommand() *cobra.Command {
 	if _, err := os.Stat(defaultPullSecretFile); os.IsNotExist(err) {
 		defaultPullSecretFile = ""
 	} else if err != nil {
-		log.WithError(err).Errorf("%v can not be used", defaultPullSecretFile)
+		opt.log.WithError(err).Errorf("%v can not be used", defaultPullSecretFile)
 	}
 
 	cmd := &cobra.Command{
@@ -222,28 +229,32 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=ovirt --ovirt-api-vip 192.168.1.2
 		Run: func(cmd *cobra.Command, args []string) {
 			log.SetLevel(log.InfoLevel)
 			if err := opt.Complete(cmd, args); err != nil {
-				log.WithError(err).Fatal("Error")
+				opt.log.WithError(err).Fatal("Error")
 			}
 			if err := opt.Validate(cmd); err != nil {
-				log.WithError(err).Fatal("Error")
+				opt.log.WithError(err).Fatal("Error")
 			}
 			err := opt.Run()
 			if err != nil {
-				log.WithError(err).Fatal("Error")
+				opt.log.WithError(err).Fatal("Error")
 			}
 		},
 	}
 
 	flags := cmd.Flags()
-	flags.StringVar(&opt.Cloud, "cloud", cloudAWS, "Cloud provider: aws(default)|azure|gcp|openstack)")
+	flags.StringVar(&opt.Cloud, "cloud", cloudAWS, "Cloud provider: aws|azure|gcp|openstack")
 	flags.StringVarP(&opt.Namespace, "namespace", "n", "", "Namespace to create cluster deployment in")
 	flags.StringVar(&opt.SSHPrivateKeyFile, "ssh-private-key-file", "", "file name containing private key contents")
 	flags.StringVar(&opt.SSHPublicKeyFile, "ssh-public-key-file", defaultSSHPublicKeyFile, "file name of SSH public key for cluster")
 	flags.StringVar(&opt.SSHPublicKey, "ssh-public-key", "", "SSH public key for cluster")
 	flags.StringVar(&opt.BaseDomain, "base-domain", "new-installer.openshift.com", "Base domain for the cluster")
 	flags.StringVar(&opt.PullSecret, "pull-secret", "", "Pull secret for cluster. Takes precedence over pull-secret-file.")
-	flags.StringVar(&opt.DeleteAfter, "delete-after", "", "Delete this cluster after the given duration. (i.e. 8h)")
+	flags.StringVar(&opt.DeleteAfter, "delete-after", "", "Delete this cluster after the given duration. (e.g. 8h)")
+	flags.StringVar(&opt.HibernateAfter, "hibernate-after", "", "Automatically hibernate the cluster whenever it has been running for the given duration")
 	flags.StringVar(&opt.PullSecretFile, "pull-secret-file", defaultPullSecretFile, "Pull secret file for cluster")
+	flags.StringVar(&opt.BoundServiceAccountSigningKeyFile, "bound-service-account-signing-key-file", "", "Private service account signing key (often created with ccoutil create key-pair)")
+	flags.BoolVar(&opt.CredentialsModeManual, "credentials-mode-manual", false, "Configure the Cloud Credential Operator in the target cluster to Manual mode. Implies the use of --manifests-dir to inject custom Secrets for all CredentialsRequests in the cluster.")
+
 	flags.StringVar(&opt.CredsFile, "creds-file", "", "Cloud credentials file (defaults vary depending on cloud)")
 	flags.StringVar(&opt.ClusterImageSet, "image-set", "", "Cluster image set to use for this cluster deployment")
 	flags.StringVar(&opt.ReleaseImage, "release-image", "", "Release image to use for installing this cluster deployment")
@@ -251,7 +262,7 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=ovirt --ovirt-api-vip 192.168.1.2
 	flags.StringVar(&opt.ServingCert, "serving-cert", "", "Serving certificate for control plane and routes")
 	flags.StringVar(&opt.ServingCertKey, "serving-cert-key", "", "Serving certificate key for control plane and routes")
 	flags.BoolVar(&opt.ManageDNS, "manage-dns", false, "Manage this cluster's DNS. This is only available for AWS and GCP.")
-	flags.BoolVar(&opt.UseClusterImageSet, "use-image-set", true, "If true(default), use a cluster image set for this cluster")
+	flags.BoolVar(&opt.UseClusterImageSet, "use-image-set", true, "If true, use a cluster image set for this cluster")
 	flags.StringVarP(&opt.Output, "output", "o", "", "Output of this command (nothing will be created on cluster). Valid values: yaml,json")
 	flags.BoolVar(&opt.IncludeSecrets, "include-secrets", true, "Include secrets along with ClusterDeployment")
 	flags.BoolVar(&opt.InstallOnce, "install-once", false, "Run the install only one time and fail if not successful")
@@ -263,7 +274,11 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=ovirt --ovirt-api-vip 192.168.1.2
 	flags.StringVar(&opt.MachineNetwork, "machine-network", "10.0.0.0/16", "Cluster's MachineNetwork to pass to the installer")
 	flags.StringVar(&opt.Region, "region", "", "Region to which to install the cluster. This is only relevant to AWS, Azure, and GCP.")
 	flags.StringSliceVarP(&opt.Labels, "labels", "l", nil, "Label to apply to the ClusterDeployment (key=val)")
+	flags.StringSliceVarP(&opt.Annotations, "annotations", "a", nil, "Annotation to apply to the ClusterDeployment (key=val)")
 	flags.BoolVar(&opt.SkipMachinePools, "skip-machine-pools", false, "Skip generation of Hive MachinePools for day 2 MachineSet management")
+	flags.BoolVar(&opt.CentralMachineManagement, "central-machine-mgmt", false, "Enable central machine management for cluster")
+	flags.BoolVar(&opt.Internal, "internal", false, `When set, it configures the install-config.yaml's publish field to Internal.
+OpenShift Installer publishes all the services of the cluster like API server and ingress to internal network and not the Internet.`)
 
 	// Flags related to adoption.
 	flags.BoolVar(&opt.Adopt, "adopt", false, "Enable adoption mode for importing a pre-existing cluster into Hive. Will require additional flags for adoption info.")
@@ -275,6 +290,7 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=ovirt --ovirt-api-vip 192.168.1.2
 
 	// AWS flags
 	flags.StringSliceVar(&opt.AWSUserTags, "aws-user-tags", nil, "Additional tags to add to resources. Must be in the form \"key=value\"")
+	flags.BoolVar(&opt.AWSPrivateLink, "aws-private-link", false, "Enables access to cluster using AWS PrivateLink")
 
 	// Azure flags
 	flags.StringVar(&opt.AzureBaseDomainResourceGroupName, "azure-base-domain-resource-group-name", "os4-common", "Resource group where the azure DNS zone for the base domain is found")
@@ -285,6 +301,7 @@ create-cluster CLUSTER_DEPLOYMENT_NAME --cloud=ovirt --ovirt-api-vip 192.168.1.2
 	flags.StringVar(&opt.OpenStackMasterFlavor, "openstack-master-flavor", "ci.m4.xlarge", "Compute flavor to use for master nodes")
 	flags.StringVar(&opt.OpenStackComputeFlavor, "openstack-compute-flavor", "m1.large", "Compute flavor to use for worker nodes")
 	flags.StringVar(&opt.OpenStackAPIFloatingIP, "openstack-api-floating-ip", "", "Floating IP address to use for cluster's API")
+	flags.StringVar(&opt.OpenStackIngressFloatingIP, "openstack-ingress-floating-ip", "", "Floating IP address to use for cluster's Ingress service")
 
 	// vSphere flags
 	flags.StringVar(&opt.VSphereVCenter, "vsphere-vcenter", "", "Domain name or IP address of the vCenter")
@@ -326,6 +343,14 @@ func (o *Options) Complete(cmd *cobra.Command, args []string) error {
 		}
 	}
 
+	if o.HibernateAfter != "" {
+		dur, err := time.ParseDuration(o.HibernateAfter)
+		if err != nil {
+			return errors.Wrapf(err, "unable to parse HibernateAfter duration")
+		}
+		o.HibernateAfterDur = &dur
+	}
+
 	return nil
 }
 
@@ -333,35 +358,45 @@ func (o *Options) Complete(cmd *cobra.Command, args []string) error {
 func (o *Options) Validate(cmd *cobra.Command) error {
 	if len(o.Output) > 0 && o.Output != "yaml" && o.Output != "json" {
 		cmd.Usage()
-		log.Info("Invalid value for output. Valid values are: yaml, json.")
+		o.log.Info("Invalid value for output. Valid values are: yaml, json.")
 		return fmt.Errorf("invalid output")
 	}
 	if !o.UseClusterImageSet && len(o.ClusterImageSet) > 0 {
 		cmd.Usage()
-		log.Info("If not using cluster image sets, do not specify the name of one")
+		o.log.Info("If not using cluster image sets, do not specify the name of one")
 		return fmt.Errorf("invalid option")
 	}
 	if len(o.ServingCert) > 0 && len(o.ServingCertKey) == 0 {
 		cmd.Usage()
-		log.Info("If specifying a serving certificate, specify a valid serving certificate key")
+		o.log.Info("If specifying a serving certificate, specify a valid serving certificate key")
 		return fmt.Errorf("invalid serving cert")
 	}
 	if !validClouds[o.Cloud] {
 		cmd.Usage()
-		log.Infof("Unsupported cloud: %s", o.Cloud)
+		o.log.Infof("Unsupported cloud: %s", o.Cloud)
 		return fmt.Errorf("unsupported cloud: %s", o.Cloud)
 	}
 	if o.Cloud == cloudOpenStack {
 		if o.OpenStackAPIFloatingIP == "" {
-			log.Info("Missing openstack-api-floating-ip parameter")
+			o.log.Info("Missing openstack-api-floating-ip parameter")
 			return fmt.Errorf("Missing openstack-api-floating-ip parameter")
 		}
 		if o.OpenStackCloud == "" {
-			log.Info("Missing openstack-cloud parameter")
+			o.log.Info("Missing openstack-cloud parameter")
 			return fmt.Errorf("Missing openstack-cloud parameter")
 		}
 	}
 
+	if o.CredentialsModeManual {
+		if o.ManifestsDir == "" {
+			return fmt.Errorf("--credentials-mode-manual requires --manifests-dir containing custom Secrets with manually provisioned credentials")
+		}
+	}
+
+	if o.AWSPrivateLink && o.Cloud != cloudAWS {
+		return fmt.Errorf("--aws-private-link can only be enabled for AWS cloud platform")
+	}
+
 	if o.Adopt {
 		if o.AdoptAdminKubeConfig == "" || o.AdoptInfraID == "" || o.AdoptClusterID == "" {
 			return fmt.Errorf("must specify the following options when using --adopt: --adopt-admin-kube-config, --adopt-infra-id, --adopt-cluster-id")
@@ -395,6 +430,12 @@ func (o *Options) Validate(cmd *cobra.Command) error {
 			return fmt.Errorf("unable to parse key=value label: %s", ls)
 		}
 	}
+	for _, ls := range o.Annotations {
+		tokens := strings.Split(ls, "=")
+		if len(tokens) != 2 {
+			return fmt.Errorf("unable to parse key=value annotation: %s", ls)
+		}
+	}
 	return nil
 }
 
@@ -418,21 +459,21 @@ func (o *Options) Run() error {
 		printObjects(objs, scheme.Scheme, printer)
 		return err
 	}
-	rh, err := o.getResourceHelper()
+	rh, err := utils.GetResourceHelper(o.log)
 	if err != nil {
 		return err
 	}
 	if len(o.Namespace) == 0 {
-		o.Namespace, err = o.defaultNamespace()
+		o.Namespace, err = utils.DefaultNamespace()
 		if err != nil {
-			log.Error("Cannot determine default namespace")
+			o.log.Error("Cannot determine default namespace")
 			return err
 		}
 	}
 	for _, obj := range objs {
 		accessor, err := meta.Accessor(obj)
 		if err != nil {
-			log.WithError(err).Errorf("Cannot create accessor for object of type %T", obj)
+			o.log.WithError(err).Errorf("Cannot create accessor for object of type %T", obj)
 			return err
 		}
 		accessor.SetNamespace(o.Namespace)
@@ -444,27 +485,10 @@ func (o *Options) Run() error {
 	return nil
 }
 
-func (o *Options) defaultNamespace() (string, error) {
-	rules := clientcmd.NewDefaultClientConfigLoadingRules()
-	kubeconfig := clientcmd.NewNonInteractiveDeferredLoadingClientConfig(rules, &clientcmd.ConfigOverrides{})
-	ns, _, err := kubeconfig.Namespace()
-	return ns, err
-}
-
-func (o *Options) getResourceHelper() (resource.Helper, error) {
-	cfg, err := config.GetConfig()
-	if err != nil {
-		log.WithError(err).Error("Cannot get client config")
-		return nil, err
-	}
-	helper := resource.NewHelperFromRESTConfig(cfg, log.WithField("command", "create-cluster"))
-	return helper, nil
-}
-
 // GenerateObjects generates resources for a new cluster deployment
 func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 
-	pullSecret, err := o.getPullSecret()
+	pullSecret, err := utils.GetPullSecret(o.log, o.PullSecret, o.PullSecretFile)
 	if err != nil {
 		return nil, err
 	}
@@ -498,22 +522,31 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 		labels[tokens[0]] = tokens[1]
 	}
 
+	annotations := map[string]string{}
+	for _, ls := range o.Annotations {
+		tokens := strings.Split(ls, "=")
+		annotations[tokens[0]] = tokens[1]
+	}
+
 	builder := &clusterresource.Builder{
-		Name:                  o.Name,
-		Namespace:             o.Namespace,
-		WorkerNodesCount:      o.WorkerNodesCount,
-		PullSecret:            pullSecret,
-		SSHPrivateKey:         sshPrivateKey,
-		SSHPublicKey:          sshPublicKey,
-		InstallOnce:           o.InstallOnce,
-		BaseDomain:            o.BaseDomain,
-		ManageDNS:             o.ManageDNS,
-		DeleteAfter:           o.DeleteAfter,
-		Labels:                labels,
-		InstallerManifests:    manifestFileData,
-		MachineNetwork:        o.MachineNetwork,
-		SkipMachinePools:      o.SkipMachinePools,
-		AdditionalTrustBundle: additionalTrustBundle,
+		Name:                     o.Name,
+		Namespace:                o.Namespace,
+		WorkerNodesCount:         o.WorkerNodesCount,
+		PullSecret:               pullSecret,
+		SSHPrivateKey:            sshPrivateKey,
+		SSHPublicKey:             sshPublicKey,
+		InstallOnce:              o.InstallOnce,
+		BaseDomain:               o.BaseDomain,
+		ManageDNS:                o.ManageDNS,
+		DeleteAfter:              o.DeleteAfter,
+		HibernateAfter:           o.HibernateAfterDur,
+		Labels:                   labels,
+		Annotations:              annotations,
+		InstallerManifests:       manifestFileData,
+		MachineNetwork:           o.MachineNetwork,
+		SkipMachinePools:         o.SkipMachinePools,
+		AdditionalTrustBundle:    additionalTrustBundle,
+		CentralMachineManagement: o.CentralMachineManagement,
 	}
 	if o.Adopt {
 		kubeconfigBytes, err := ioutil.ReadFile(o.AdoptAdminKubeConfig)
@@ -527,6 +560,13 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 		builder.AdoptAdminUsername = o.AdoptAdminUsername
 		builder.AdoptAdminPassword = o.AdoptAdminPassword
 	}
+	if len(o.BoundServiceAccountSigningKeyFile) != 0 {
+		signingKey, err := ioutil.ReadFile(o.BoundServiceAccountSigningKeyFile)
+		if err != nil {
+			return nil, fmt.Errorf("error reading %s: %v", o.BoundServiceAccountSigningKeyFile, err)
+		}
+		builder.BoundServiceAccountSigningKey = string(signingKey)
+	}
 
 	switch o.Cloud {
 	case cloudAWS:
@@ -551,12 +591,13 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 			SecretAccessKey: secretAccessKey,
 			UserTags:        userTags,
 			Region:          o.Region,
+			PrivateLink:     o.AWSPrivateLink,
 		}
 		builder.CloudBuilder = awsProvider
 	case cloudAzure:
 		creds, err := azurecredutil.GetCreds(o.CredsFile)
 		if err != nil {
-			log.WithError(err).Error("Failed to read in Azure credentials")
+			o.log.WithError(err).Error("Failed to read in Azure credentials")
 			return nil, err
 		}
 
@@ -594,6 +635,7 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 			ComputeFlavor:     o.OpenStackComputeFlavor,
 			MasterFlavor:      o.OpenStackMasterFlavor,
 			APIFloatingIP:     o.OpenStackAPIFloatingIP,
+			IngressFloatingIP: o.OpenStackIngressFloatingIP,
 		}
 		builder.CloudBuilder = openStackProvider
 	case cloudVSphere:
@@ -641,7 +683,7 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 			vSphereDatastore = o.VSphereDefaultDataStore
 		}
 		if vSphereDatastore == "" {
-			return nil, fmt.Errorf("must provide --vsphere-datastore or set %s env var", constants.VSphereDataStoreEnvVar)
+			return nil, fmt.Errorf("must provide --vsphere-default-datastore or set %s env var", constants.VSphereDataStoreEnvVar)
 		}
 
 		vSphereVCenter := os.Getenv(constants.VSphereVCenterEnvVar)
@@ -695,6 +737,10 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 		builder.SkipMachinePools = true
 	}
 
+	if o.Internal {
+		builder.PublishStrategy = "Internal"
+	}
+
 	if len(o.ServingCert) != 0 {
 		servingCert, err := ioutil.ReadFile(o.ServingCert)
 		if err != nil {
@@ -730,26 +776,6 @@ func (o *Options) GenerateObjects() ([]runtime.Object, error) {
 	return result, nil
 }
 
-func (o *Options) getPullSecret() (string, error) {
-	pullSecret := os.Getenv("PULL_SECRET")
-	if len(pullSecret) > 0 {
-		return pullSecret, nil
-	}
-	if len(o.PullSecret) > 0 {
-		return o.PullSecret, nil
-	}
-	if len(o.PullSecretFile) > 0 {
-		data, err := ioutil.ReadFile(o.PullSecretFile)
-		if err != nil {
-			log.Error("Cannot read pull secret file")
-			return "", err
-		}
-		pullSecret = strings.TrimSpace(string(data))
-		return pullSecret, nil
-	}
-	return "", nil
-}
-
 func (o *Options) getSSHPublicKey() (string, error) {
 	sshPublicKey := os.Getenv("PUBLIC_SSH_KEY")
 	if len(sshPublicKey) > 0 {
@@ -761,14 +787,14 @@ func (o *Options) getSSHPublicKey() (string, error) {
 	if len(o.SSHPublicKeyFile) > 0 {
 		data, err := ioutil.ReadFile(o.SSHPublicKeyFile)
 		if err != nil {
-			log.Error("Cannot read SSH public key file")
+			o.log.Error("Cannot read SSH public key file")
 			return "", err
 		}
 		sshPublicKey = strings.TrimSpace(string(data))
 		return sshPublicKey, nil
 	}
 
-	log.Error("Cannot determine SSH key to use")
+	o.log.Error("Cannot determine SSH key to use")
 	return "", nil
 }
 
@@ -776,13 +802,13 @@ func (o *Options) getSSHPrivateKey() (string, error) {
 	if len(o.SSHPrivateKeyFile) > 0 {
 		data, err := ioutil.ReadFile(o.SSHPrivateKeyFile)
 		if err != nil {
-			log.Error("Cannot read SSH private key file")
+			o.log.Error("Cannot read SSH private key file")
 			return "", err
 		}
 		sshPrivateKey := strings.TrimSpace(string(data))
 		return sshPrivateKey, nil
 	}
-	log.Debug("No private SSH key file provided")
+	o.log.Debug("No private SSH key file provided")
 	return "", nil
 }
 
@@ -790,17 +816,17 @@ func (o *Options) getAdditionalTrustBundle() (string, error) {
 	if len(o.AdditionalTrustBundle) > 0 {
 		data, err := ioutil.ReadFile(o.AdditionalTrustBundle)
 		if err != nil {
-			log.Error("Cannot read AdditionalTrustBundle file")
+			o.log.Error("Cannot read AdditionalTrustBundle file")
 			return "", err
 		}
 		if err := validate.CABundle(string(data)); err != nil {
-			log.Error("AdditionalTrustBundle is not valid")
+			o.log.Error("AdditionalTrustBundle is not valid")
 			return "", err
 		}
 		additionalTrustBundle := string(data)
 		return additionalTrustBundle, nil
 	}
-	log.Debug("No AdditionalTrustBundle provided")
+	o.log.Debug("No AdditionalTrustBundle provided")
 	return "", nil
 }
 
@@ -842,7 +868,7 @@ func (o *Options) configureImages(generator *clusterresource.Builder) (*hivev1.C
 			return nil, fmt.Errorf("specify either a release image or a release image source")
 		}
 		var err error
-		o.ReleaseImage, err = determineReleaseImageFromSource(o.ReleaseImageSource)
+		o.ReleaseImage, err = utils.DetermineReleaseImageFromSource(o.ReleaseImageSource)
 		if err != nil {
 			return nil, fmt.Errorf("cannot determine release image: %v", err)
 		}
@@ -965,25 +991,3 @@ func printObjects(objects []runtime.Object, scheme *runtime.Scheme, printer prin
 		typeSetterPrinter.PrintObj(list, os.Stdout)
 	}
 }
-
-type releasePayload struct {
-	PullSpec string `json:"pullSpec"`
-}
-
-func determineReleaseImageFromSource(sourceURL string) (string, error) {
-	resp, err := http.Get(sourceURL)
-	if err != nil {
-		return "", err
-	}
-	defer resp.Body.Close()
-	data, err := ioutil.ReadAll(resp.Body)
-	if err != nil {
-		return "", err
-	}
-	payload := &releasePayload{}
-	err = json.Unmarshal(data, payload)
-	if err != nil {
-		return "", err
-	}
-	return payload.PullSpec, nil
-}
