@@ -12,6 +12,8 @@ import (
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/client-go/util/flowcontrol"
+	"k8s.io/client-go/util/workqueue"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
@@ -19,14 +21,15 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	"github.com/openshift/hive/pkg/constants"
 	hivemetrics "github.com/openshift/hive/pkg/controller/metrics"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	"github.com/openshift/hive/pkg/resource"
 )
 
 const (
-	ControllerName                = "clusterclaim"
+	ControllerName                = hivev1.ClusterClaimControllerName
 	finalizer                     = "hive.openshift.io/claim"
 	hiveClaimOwnerRoleName        = "hive-claim-owner"
 	hiveClaimOwnerRoleBindingName = "hive-claim-owner"
@@ -35,22 +38,32 @@ const (
 // Add creates a new ClusterClaim Controller and adds it to the Manager with default RBAC. The Manager will set fields on the Controller
 // and Start it when the Manager is Started.
 func Add(mgr manager.Manager) error {
-	return AddToManager(mgr, NewReconciler(mgr))
+	logger := log.WithField("controller", ControllerName)
+	concurrentReconciles, clientRateLimiter, queueRateLimiter, err := controllerutils.GetControllerConfig(mgr.GetClient(), ControllerName)
+	if err != nil {
+		logger.WithError(err).Error("could not get controller configurations")
+		return err
+	}
+	return AddToManager(mgr, NewReconciler(mgr, clientRateLimiter), concurrentReconciles, queueRateLimiter)
 }
 
 // NewReconciler returns a new ReconcileClusterClaim
-func NewReconciler(mgr manager.Manager) *ReconcileClusterClaim {
+func NewReconciler(mgr manager.Manager, rateLimiter flowcontrol.RateLimiter) *ReconcileClusterClaim {
 	logger := log.WithField("controller", ControllerName)
 	return &ReconcileClusterClaim{
-		Client: controllerutils.NewClientWithMetricsOrDie(mgr, ControllerName),
+		Client: controllerutils.NewClientWithMetricsOrDie(mgr, ControllerName, &rateLimiter),
 		logger: logger,
 	}
 }
 
 // AddToManager adds a new Controller to mgr with r as the reconcile.Reconciler
-func AddToManager(mgr manager.Manager, r *ReconcileClusterClaim) error {
+func AddToManager(mgr manager.Manager, r *ReconcileClusterClaim, concurrentReconciles int, rateLimiter workqueue.RateLimiter) error {
 	// Create a new controller
-	c, err := controller.New("clusterclaim-controller", mgr, controller.Options{Reconciler: r, MaxConcurrentReconciles: controllerutils.GetConcurrentReconciles()})
+	c, err := controller.New("clusterclaim-controller", mgr, controller.Options{
+		Reconciler:              r,
+		MaxConcurrentReconciles: concurrentReconciles,
+		RateLimiter:             rateLimiter,
+	})
 	if err != nil {
 		return err
 	}
@@ -63,30 +76,21 @@ func AddToManager(mgr manager.Manager, r *ReconcileClusterClaim) error {
 	// Watch for changes to ClusterDeployment
 	if err := c.Watch(
 		&source.Kind{Type: &hivev1.ClusterDeployment{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: handler.ToRequestsFunc(requestsForClusterDeployment),
-		},
-	); err != nil {
+		handler.EnqueueRequestsFromMapFunc(requestsForClusterDeployment)); err != nil {
 		return err
 	}
 
 	// Watch for changes to the hive-claim-owner Role
 	if err := c.Watch(
 		&source.Kind{Type: &rbacv1.Role{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: requestsForRBACResources(r.Client, hiveClaimOwnerRoleName, r.logger),
-		},
-	); err != nil {
+		handler.EnqueueRequestsFromMapFunc(requestsForRBACResources(r.Client, hiveClaimOwnerRoleName, r.logger))); err != nil {
 		return err
 	}
 
 	// Watch for changes to the hive-claim-owner RoleBinding
 	if err := c.Watch(
 		&source.Kind{Type: &rbacv1.Role{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: requestsForRBACResources(r.Client, hiveClaimOwnerRoleBindingName, r.logger),
-		},
-	); err != nil {
+		handler.EnqueueRequestsFromMapFunc(requestsForRBACResources(r.Client, hiveClaimOwnerRoleBindingName, r.logger))); err != nil {
 		return err
 	}
 
@@ -106,8 +110,8 @@ func claimForClusterDeployment(cd *hivev1.ClusterDeployment) *types.NamespacedNa
 	}
 }
 
-func requestsForClusterDeployment(o handler.MapObject) []reconcile.Request {
-	cd, ok := o.Object.(*hivev1.ClusterDeployment)
+func requestsForClusterDeployment(o client.Object) []reconcile.Request {
+	cd, ok := o.(*hivev1.ClusterDeployment)
 	if !ok {
 		return nil
 	}
@@ -118,12 +122,12 @@ func requestsForClusterDeployment(o handler.MapObject) []reconcile.Request {
 	return []reconcile.Request{{NamespacedName: *claim}}
 }
 
-func requestsForRBACResources(c client.Client, resourceName string, logger log.FieldLogger) handler.ToRequestsFunc {
-	return func(o handler.MapObject) []reconcile.Request {
-		if o.Meta.GetName() != resourceName {
+func requestsForRBACResources(c client.Client, resourceName string, logger log.FieldLogger) handler.MapFunc {
+	return func(o client.Object) []reconcile.Request {
+		if o.GetName() != resourceName {
 			return nil
 		}
-		clusterName := o.Meta.GetNamespace()
+		clusterName := o.GetNamespace()
 		cd := &hivev1.ClusterDeployment{}
 		if err := c.Get(context.Background(), client.ObjectKey{Namespace: clusterName, Name: clusterName}, cd); err != nil {
 			logger.WithError(err).Log(controllerutils.LogLevel(err), "failed to get ClusterDeployment for RBAC resource")
@@ -146,16 +150,11 @@ type ReconcileClusterClaim struct {
 }
 
 // Reconcile reconciles a ClusterClaim.
-func (r *ReconcileClusterClaim) Reconcile(request reconcile.Request) (reconcile.Result, error) {
-	start := time.Now()
-	logger := r.logger.WithField("clusterClaim", request.NamespacedName)
-
+func (r *ReconcileClusterClaim) Reconcile(ctx context.Context, request reconcile.Request) (result reconcile.Result, returnErr error) {
+	logger := controllerutils.BuildControllerLogger(ControllerName, "clusterClaim", request.NamespacedName)
 	logger.Infof("reconciling cluster claim")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName).Observe(dur.Seconds())
-		logger.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, logger)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the ClusterClaim instance
 	claim := &hivev1.ClusterClaim{}
@@ -192,6 +191,47 @@ func (r *ReconcileClusterClaim) Reconcile(request reconcile.Request) (reconcile.
 
 	logger = logger.WithField("cluster", clusterName)
 
+	poolLifetime, err := r.clusterPoolLifetimeForClaim(claim, logger)
+	if err != nil {
+		logger.Log(controllerutils.LogLevel(err), "error getting cluster pool lifetime")
+		return reconcile.Result{}, err
+	}
+	lifetime := getClaimLifetime(poolLifetime, claim.Spec.Lifetime)
+
+	if (lifetime != nil) != (claim.Status.Lifetime != nil) ||
+		lifetime != nil && claim.Status.Lifetime != nil && lifetime.Duration != claim.Status.Lifetime.Duration {
+		claim.Status.Lifetime = lifetime
+		if err := r.Status().Update(context.Background(), claim); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "could not update ClusterClaim lifetime")
+			return reconcile.Result{}, errors.Wrap(err, "could not update ClusterClaim lifetime")
+		}
+	}
+
+	// Delete ClusterClaim after its lifetime elapses
+	if lifetime != nil {
+		logger.WithField("lifetime", lifetime).Debug("checking whether lifetime of ClusterClaim has elapsed")
+		pendingCond := controllerutils.FindClusterClaimCondition(claim.Status.Conditions, hivev1.ClusterClaimPendingCondition)
+		if pendingCond != nil && pendingCond.Status == corev1.ConditionFalse {
+			if timeSinceAssigned := time.Since(pendingCond.LastTransitionTime.Time); timeSinceAssigned >= lifetime.Duration {
+				logger.WithField("timeSinceAssigned", timeSinceAssigned).
+					WithField("lifetime", lifetime).
+					Info("deleting ClusterClaim because its lifetime has elapsed")
+				if err := r.Delete(context.Background(), claim); err != nil {
+					logger.WithError(err).Log(controllerutils.LogLevel(err), "could not delete ClusterClaim")
+					return reconcile.Result{}, errors.Wrap(err, "could not delete ClusterClaim")
+				}
+				return reconcile.Result{}, nil
+			}
+			defer func() {
+				result, returnErr = controllerutils.EnsureRequeueAtLeastWithin(
+					lifetime.Duration-time.Since(pendingCond.LastTransitionTime.Time),
+					result,
+					returnErr,
+				)
+			}()
+		}
+	}
+
 	cd := &hivev1.ClusterDeployment{}
 	switch err := r.Get(context.Background(), client.ObjectKey{Namespace: clusterName, Name: clusterName}, cd); {
 	case apierrors.IsNotFound(err):
@@ -211,6 +251,47 @@ func (r *ReconcileClusterClaim) Reconcile(request reconcile.Request) (reconcile.
 	}
 }
 
+// getClaimLifetime returns the lifetime for a claim taking into account the lifetime set on the pool
+// and the claim.
+// if no lifetime is set on the claim, the default lifetime for the pool is used if set.
+// if maximum lifetime for the pool, the lifetime is minimum of this maximum and lifetime from above.
+func getClaimLifetime(poolLifetime *hivev1.ClusterPoolClaimLifetime, claimLifetime *metav1.Duration) *metav1.Duration {
+	var lifetime *metav1.Duration
+	if poolLifetime != nil && poolLifetime.Default != nil {
+		lifetime = poolLifetime.Default
+	}
+	if claimLifetime != nil {
+		lifetime = claimLifetime
+	}
+	if poolLifetime != nil && poolLifetime.Maximum != nil {
+		if lifetime == nil ||
+			(poolLifetime.Maximum.Duration < lifetime.Duration) {
+			lifetime = poolLifetime.Maximum
+		}
+
+	}
+	return lifetime
+}
+
+// clusterPoolLifetimeForClaim returns the default and max lifetimes for the cluster pool the claim belongs to.
+func (r *ReconcileClusterClaim) clusterPoolLifetimeForClaim(claim *hivev1.ClusterClaim, logger log.FieldLogger) (*hivev1.ClusterPoolClaimLifetime, error) {
+	// Fetch the ClusterPool instance
+	clp := &hivev1.ClusterPool{}
+	// claims exists in the same namespace as the pool
+	key := client.ObjectKey{Namespace: claim.Namespace, Name: claim.Spec.ClusterPoolName}
+	err := r.Get(context.TODO(), key, clp)
+	if apierrors.IsNotFound(err) {
+		logger.WithField("pool", key).WithField("claim", claim.Name).Info("cluster pool no longer exists")
+		// since there is no pool no lifetime can be extracted. this is a valid state.
+		return nil, nil
+	}
+	if err != nil {
+		log.WithError(err).Error("error reading cluster pool")
+		return nil, errors.Wrap(err, "failed to get the pool")
+	}
+	return clp.Spec.ClaimLifetime, nil
+}
+
 func (r *ReconcileClusterClaim) reconcileDeletedClaim(claim *hivev1.ClusterClaim, logger log.FieldLogger) (reconcile.Result, error) {
 	if !controllerutils.HasFinalizer(claim, finalizer) {
 		return reconcile.Result{}, nil
@@ -274,10 +355,15 @@ func (r *ReconcileClusterClaim) cleanupResources(claim *hivev1.ClusterClaim, log
 	}
 
 	// Delete ClusterDeployment
-	if cd.DeletionTimestamp == nil {
+	toRemove := controllerutils.IsClaimedClusterMarkedForRemoval(cd)
+	if cd.DeletionTimestamp == nil && !toRemove {
 		logger.Info("deleting clusterDeployment")
-		if err := r.Delete(context.Background(), cd); err != nil {
-			logger.WithError(err).Log(controllerutils.LogLevel(err), "error deleting ClusterDeployment")
+		if cd.Annotations == nil {
+			cd.Annotations = map[string]string{}
+		}
+		cd.Annotations[constants.ClusterClaimRemoveClusterAnnotation] = "true"
+		if err := r.Update(context.Background(), cd); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "error updating ClusterDeployment to mark it for deletion")
 			return err
 		}
 	}
@@ -321,15 +407,45 @@ func (r *ReconcileClusterClaim) reconcileForExistingAssignment(claim *hivev1.Clu
 	if err := r.createRBAC(claim, cd, logger); err != nil {
 		return reconcile.Result{}, err
 	}
-	conds, changed := controllerutils.SetClusterClaimConditionWithChangeCheck(
-		claim.Status.Conditions,
+	var statusChanged bool
+	var changed bool
+	conds := claim.Status.Conditions
+
+	conds, changed = controllerutils.SetClusterClaimConditionWithChangeCheck(
+		conds,
 		hivev1.ClusterClaimPendingCondition,
 		corev1.ConditionFalse,
 		"ClusterClaimed",
 		"Cluster claimed",
 		controllerutils.UpdateConditionIfReasonOrMessageChange,
 	)
-	if changed {
+	statusChanged = statusChanged || changed
+
+	hc := controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.ClusterHibernatingCondition)
+	if hc == nil || hc.Status == corev1.ConditionFalse {
+		conds, changed = controllerutils.SetClusterClaimConditionWithChangeCheck(
+			conds,
+			hivev1.ClusterRunningCondition,
+			corev1.ConditionTrue,
+			"Running",
+			"Cluster is running",
+			controllerutils.UpdateConditionIfReasonOrMessageChange,
+		)
+		statusChanged = statusChanged || changed
+	} else {
+		log.Debug("waiting for cluster to be running")
+		conds, changed = controllerutils.SetClusterClaimConditionWithChangeCheck(
+			conds,
+			hivev1.ClusterRunningCondition,
+			corev1.ConditionFalse,
+			"Resuming",
+			"Waiting for cluster to be running",
+			controllerutils.UpdateConditionIfReasonOrMessageChange,
+		)
+		statusChanged = statusChanged || changed
+	}
+	if statusChanged {
+		log.Debug("conditions changed, updating claim status")
 		claim.Status.Conditions = conds
 		if err := r.Status().Update(context.Background(), claim); err != nil {
 			logger.WithError(err).Log(controllerutils.LogLevel(err), "could not update status of ClusterClaim")
