@@ -6,6 +6,7 @@ import (
 	"strings"
 
 	log "github.com/sirupsen/logrus"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/arn"
@@ -15,7 +16,7 @@ import (
 
 	corev1 "k8s.io/api/core/v1"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
 	awsclient "github.com/openshift/hive/pkg/awsclient"
 	"github.com/openshift/hive/pkg/constants"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
@@ -46,12 +47,13 @@ type AWSActuator struct {
 	dnsZone *hivev1.DNSZone
 }
 
-type awsClientBuilderType func(secret *corev1.Secret, region string) (awsclient.Client, error)
+type awsClientBuilderType func(client.Client, awsclient.Options) (awsclient.Client, error)
 
 // NewAWSActuator creates a new AWSActuator object. A new AWSActuator is expected to be created for each controller sync.
 func NewAWSActuator(
 	logger log.FieldLogger,
-	secret *corev1.Secret,
+	kubeClient client.Client,
+	credentials awsclient.CredentialsSource,
 	dnsZone *hivev1.DNSZone,
 	awsClientBuilder awsClientBuilderType,
 ) (*AWSActuator, error) {
@@ -59,7 +61,10 @@ func NewAWSActuator(
 	if region == "" {
 		region = constants.AWSRoute53Region
 	}
-	awsClient, err := awsClientBuilder(secret, region)
+	awsClient, err := awsClientBuilder(kubeClient, awsclient.Options{
+		Region:            region,
+		CredentialsSource: credentials,
+	})
 	if err != nil {
 		logger.WithError(err).Error("Error creating AWSClient")
 		return nil, err
@@ -163,8 +168,8 @@ func (a *AWSActuator) syncTags() error {
 	return nil
 }
 
-// ModifyStatus updates the DnsZone's status with AWS specific information.
-func (a *AWSActuator) ModifyStatus() error {
+// modifyStatus updates the DnsZone's status with AWS specific information.
+func (a *AWSActuator) modifyStatus() error {
 	if a.hostedZone == nil {
 		return errors.New("zoneID is unpopulated")
 	}
@@ -229,7 +234,7 @@ func (a *AWSActuator) Refresh() error {
 		a.hostedZone = resp.HostedZone
 
 		// Update dnsZone status now that we have the zoneID
-		if err := a.ModifyStatus(); err != nil {
+		if err := a.modifyStatus(); err != nil {
 			a.logger.WithError(err).Error("failed to update status after refresh")
 			return err
 		}
@@ -361,7 +366,7 @@ func (a *AWSActuator) Create() error {
 	}
 
 	a.hostedZone = hostedZone
-	if err := a.ModifyStatus(); err != nil {
+	if err := a.modifyStatus(); err != nil {
 		logger.WithError(err).Error("failed to populate DNSZone status")
 		return err
 	}
